/*
 * Copyright (C) 1999, 2000, 2001  Lorenzo Bettini, bettini@gnu.org
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include "chartranslator.h"
#include "my_sstream.h"

using namespace std;

CharTranslator::CharTranslator(TextFormatterPtr f)
  : TextFormatter(f), counter(0), reg_exp(0)
{
}

CharTranslator::~CharTranslator()
{
  if (reg_exp)
    delete reg_exp;
}

void
CharTranslator::set_translation(const std::string &to_translate,
                                const std::string &translate_into)
{
  ostringstream exp;
  exp << (translation_exp.size() ? "|" : "") << "(" << to_translate << ")";

  translation_exp += exp.str();

  ostringstream format;
  format << "(?" << ++counter << translate_into << ")";

  translation_format += format.str();
}

// s must be a NULL terminated string
const string
CharTranslator::preformat(const string &text)
{
  if (!translation_exp.size()) {
    return text;
  }

  if (! reg_exp)
    reg_exp = new boost::regex(translation_exp);

  std::ostringstream preformat_text(std::ios::out | std::ios::binary);
  std::ostream_iterator<char, char> oi(preformat_text);
  boost::regex_replace(oi, text.begin(), text.end(), *reg_exp,
                       translation_format, boost::match_default | boost::format_all);

  return preformat_text.str();
}
